#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/pool.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <stdlib.h>
#include <string.h>

typedef struct install_list {
  lm_pkg_t *packages;
  lm_pkg_t *resolving;
  size_t count;
} install_list_t;

bool install_resolve(lm_ctx_t *ctx, install_list_t *list, lm_pkg_t *pkg){
  if(pkglist_contains(list->packages, pkg))
    return true;

  if(NULL == pkg->depends)
    goto end;

  list->resolving = pkglist_add(list->resolving, pkg);

  for(int i = 0; pkg->depends[i] != NULL; i++){
    lm_pkg_t *depend = lm_ctx_find(ctx, pkg->depends[i], NULL);
    if(NULL == depend){
      lm_error_set(LM_ERR_DependNotFound, pkg->depends[i], pkg->name);
      return false;
    }

    if(lm_ctx_database_is_installed(ctx, depend, true))
      continue;

    if(pkglist_contains(list->resolving, depend)){
      lm_error_set(LM_ERR_DependNotFound, pkg->depends[i], pkg->name);
      return false;
    }

    if(!install_resolve(ctx, list, pkg))
      return false;
  }

  list->resolving = pkglist_del(list->resolving, pkg);
end:
  list->packages = pkglist_add(list->packages, pkg);
  list->count++;
  return true;
}

bool lm_ctx_install(lm_ctx_t *ctx, char *package){
  lm_pkg_t *pkg = lm_ctx_find(ctx, package, NULL);
  install_list_t list = {
    .packages  = NULL,
    .resolving = NULL,
    .count = 0,
  };
  bool ret = false;

  if(NULL == pkg)
    return ret;

  if(!install_resolve(ctx, &list, pkg))
    goto end;

  pkg = list.packages;

  while(pkg != NULL){
    if(lm_package_downloaded(pkg) && lm_package_verify(pkg))
      continue;

    if(!lm_pool_package_download(pkg->pool, pkg)){
      char *suberror = strdup(lm_strerror());
      lm_error_set(LM_ERR_InstallDownloadFail, pkg->name, suberror);
      free(suberror);
      return false;
    }

    pkg = pkg->next;
  }

  // actually install packages

end:
  pkglist_free(list.packages);
  pkglist_free(list.resolving);
  return true;
}
